# Generated by roxygen2: do not edit by hand

export(spod_available_data)
export(spod_check_files)
export(spod_cite)
export(spod_codebook)
export(spod_connect)
export(spod_convert)
export(spod_disconnect)
export(spod_download)
export(spod_get)
export(spod_get_data_dir)
export(spod_get_valid_dates)
export(spod_get_zones)
export(spod_quick_get_od)
export(spod_quick_get_zones)
export(spod_set_data_dir)
importFrom(Rdpack,reprompt)
importFrom(lifecycle,deprecated)
importFrom(memoise,memoise)
importFrom(rlang,.data)
importFrom(stats,end)
importFrom(stats,median)
importFrom(stats,start)
importFrom(utils,capture.output)
importFrom(utils,vignette)
