#' @references
#' \itemize{
#' \item **For the official website of the mobility study**: \insertRef{mitms_mobility_web}{spanishoddata}
#' \item **For v1 data methodology**: \insertRef{mitma_methodology_2020_v3}{spanishoddata}
#' \item **For v2 data methodology**: \insertRef{mitms_methodology_2022_v8}{spanishoddata}
#' \item **For the `spanishoddata` R package**: \insertRef{spanishoddata-r-pkg}{spanishoddata}
#' }
#'
#' Use \code{\link[=spod_cite]{spod_cite()}} to cite the package and the data with correct plain text, markdown, or BibTeX formats.
