#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' @importFrom lifecycle deprecated
#' @importFrom Rdpack reprompt
#' @importFrom stats end start
#' @importFrom utils capture.output
## usethis namespace: end
NULL
