#' @title Global Quiet Parameter
#'
#' @description
#' Documentation for the `quiet` parameter, used globally.
#'
#' @param quiet A `logical` value indicating whether to suppress messages. Default is `FALSE`.
#' @return Nothing. This function is just a placeholder for global quiet parameter.
#' @keywords internal
global_quiet_param <- function(quiet = FALSE){
  # this is just a placeholder for global quiet parameter
}
