---
execute: 
  eval: false
---


## Install the package {#install-package}

Install from CRAN:

```{r}
install.packages("spanishoddata")
```



<details><summary>Alternative installation and developemnt</summary>

You can also install the latest development version of the package from rOpenSpain R universe:

```{r}
install.packages("spanishoddata",
  repos = c("https://ropenspain.r-universe.dev",
    "https://cloud.r-project.org"))
```


Alternative way to install the development version from GitHub:

```{r}
if (!require("remotes")) install.packages("remotes")

remotes::install_github("rOpenSpain/spanishoddata",
  force = TRUE, dependencies = TRUE)
```

**For Developers**

To load the package locally, clone it and navigate to the root of the package in the terminal, e.g. with the following:

```bash
gh repo clone rOpenSpain/spanishoddata
code spanishoddata
# with rstudio:
rstudio spanishoddata/spanishoddata.Rproj
```

Then run the following command from the R console:

```{r}
#| eval: false
devtools::load_all()
```

</details>

You can also explore the package and the data in an interactive RStudio container right in your web browser thanks to Binder, just click [the link](https://mybinder.org/v2/gh/e-kotov/spanishoddata-playground/HEAD?urlpath=rstudio) or the button: [![Binder](https://mybinder.org/badge_logo.svg)](https://mybinder.org/v2/gh/e-kotov/spanishoddata-playground/HEAD?urlpath=rstudio). Note that the session will be limited by memory and you will only be able to work with one full day of data.
